{% extends 'admin/model/edit.html' %}
{% import 'bukuserver/lib.html' as buku with context %}

{% block tail %}
  {{ super() }}
  {{ buku.set_lang() }}
  {{ buku.brand_dbname() }}
  {{ buku.horizontal_form() }}
  {{ buku.focus() }}
{% endblock %}
