/* preventing layout overflow in tables */
table tr td:not(:first-child) {
    overflow-wrap: anywhere;
}

/* wider modals for wide data */
@media (min-width: 768px) {
    .modal-dialog {
        width: min(1000px, 80%);
    }
}

/* fixing details filter width */
#fa_filter {flex-grow: 1}

/* fixing tags input height */
.select2-container.form-control {display: flex !important}
.select2-container.form-control .select2-choices {flex-grow: 1}

.link {
    display: block;
}

.tag-list {
    display: flex;
    gap: 1px;
    flex-wrap: wrap;
}
.tag-list, .link .netloc {
    font-size: larger;
}
.tag-list a, .link .netloc, .select2-search-choice > *, .select2-result-label {
    white-space: pre;
}

.description {
    white-space: pre-wrap;
}
