{% extends 'admin/model/modals/edit.html' %}
{% import 'bukuserver/lib.html' as buku with context %}

{% block tail %}
  {{ super() }}
  {{ buku.script('bookmark.js') }}
  {{ buku.horizontal_form() }}
  {{ buku.focus('.modal-body') }}
{% endblock %}
